/* 
    qasn1editor - SignalWrapper - class for wrapping signals emitted by model/view
    Copyright(C) 2010 - Andrius da Costa Ribas

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "SignalWrapper.h"

SignalWrapper::SignalWrapper(QObject* parent) : QObject(parent)
{
    m_tag = 0;
    m_constructed = 0;
    m_definedLength = 0;
    m_rawValue = 0;
    m_universal = 0;
    m_application = 0;
    m_context = 0;
    m_private = 0;
    m_setUp = false;
}

void SignalWrapper::setTagControl(QLineEdit* tag)
{
    m_tag = tag;
}

void SignalWrapper::setConstructedControl(QCheckBox* constructed)
{
    m_constructed = constructed;
}

void SignalWrapper::setDefinedLengthControl(QCheckBox* definedLength)
{
    m_definedLength = definedLength;
}

void SignalWrapper::setRawValueControl(QPlainTextEdit* rawValue)
{
    m_rawValue = rawValue;
}

void SignalWrapper::setRadioControls(QRadioButton* _universal, QRadioButton* _application, QRadioButton* _context, QRadioButton* _private)
{
    m_application = _application;
    m_universal = _universal;
    m_context = _context;
    m_private = _private;
}

bool SignalWrapper::setupControls()
{
    m_setUp = 
    (
        m_tag != 0 &&
        m_constructed != 0 &&
        m_definedLength != 0 &&
        m_rawValue != 0 &&
        m_universal != 0 &&
        m_application != 0 &&
        m_context != 0 &&
        m_private != 0 &&
        m_model != 0
    );
    if ( m_setUp )
    {
        connect (this, SIGNAL(changeTag(QString )), m_tag, SLOT(setText(QString )));
        connect (this, SIGNAL(changeConstructed(bool )), m_constructed, SLOT(setChecked(bool )));
        connect (this, SIGNAL(changeDefinedLength(bool )), m_definedLength, SLOT(setChecked(bool )));
        connect (this, SIGNAL(changeRawValue(QString )), m_rawValue, SLOT(setPlainText(QString )));
        connect (this, SIGNAL(changeClassUniversal(bool )), m_universal, SLOT(setOn(bool )));
        connect (this, SIGNAL(changeClassApplication(bool )), m_application, SLOT(setOn(bool )));
        connect (this, SIGNAL(changeClassContext(bool )), m_context, SLOT(setOn(bool )));
        connect (this, SIGNAL(changeClassPrivate(bool )), m_private, SLOT(setOn(bool )));
    }
    return m_setUp;
}

void SignalWrapper::setModel(Asn1BerModel* model)
{
    m_model = model;
}

void SignalWrapper::changeTree(QModelIndex index)
{
    if ( !index.isValid() ) return;
    
    emit changeTag(QString("%1").arg(m_model->data(index, Asn1BerModel::TagRole).toInt()));
    emit changeConstructed(m_model->data(index, Asn1BerModel::ConstructedRole).toBool());
    emit changeDefinedLength(m_model->data(index, Asn1BerModel::DefinedLengthRole).toBool());
    emit changeRawValue(QString(m_model->data(index, Asn1BerModel::ValueRole).toByteArray().toHex()));
    switch ( (unsigned char) m_model->data(index, Asn1BerModel::ClassRole).toChar().cell() )
    {
        case CLASS_UNIVERSAL:
            emit changeClassUniversal(true);
            break;
        case CLASS_APPLICATION:
            emit changeClassApplication(true);
            break;
        case CLASS_CONTEXT:
            emit changeClassContext(true);
            break;
        case CLASS_PRIVATE:
            emit changeClassPrivate(true);
            break;
    }
}

#include <SignalWrapper.moc>