/* 
    qasn1 - modeltest - unit tests for Model class of qasn1 library
    Copyright(C) 2010 - Andrius da Costa Ribas

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <QtCore/QDebug>
#include <QtCore/QObject>
#include <QtTest/QtTest>

#include "Asn1BerParser.h"
#include "Asn1BerModel.h"

#include "modeltest.h"

void Asn1BerModelTest::initTestCase()
{
    qDebug() << "startup";
}

void Asn1BerModelTest::testFile()
{
    QFile* file = new QFile("test_1.ber");
    file->open(QIODevice::ReadOnly);
        
    Asn1BerParser* parser = new Asn1BerParser(file);
    Asn1BerNode* node;
    
    node = parser->parse();
    
    Asn1BerModel* model = new Asn1BerModel(node);
    
    QModelIndex indexA = model->index(0, 0, QModelIndex());
    QModelIndex indexB = model->index(0, 0, indexA);
    QModelIndex indexC = model->index(1, 0, indexA);
    
    QModelIndex indexD = model->index(1, 0, QModelIndex());
    QModelIndex indexE = model->index(0, 0, indexD);
    QModelIndex indexF = model->index(1, 0, indexD);
    
    QModelIndex indexG = model->index(2, 0, indexD);
    
    
    QVERIFY (model->data(indexA, Qt::DisplayRole) == QString("UNIVERSAL [16]"));
    QVERIFY (model->data(indexB, Qt::DisplayRole) == QString("CONTEXT [1048608]"));
    QVERIFY (model->data(indexC, Qt::DisplayRole) == QString("UNIVERSAL [3]"));
    QVERIFY (model->data(indexD, Qt::DisplayRole) == QString("UNIVERSAL [16]"));
    QVERIFY (model->data(indexE, Qt::DisplayRole) == QString("CONTEXT [1048608]"));
    QVERIFY (model->data(indexF, Qt::DisplayRole) == QString("UNIVERSAL [3]"));
    QVERIFY (model->data(QModelIndex(), Qt::DisplayRole) == QVariant());
    QVERIFY (model->rowCount(QModelIndex()) == 2 );
    QVERIFY (model->rowCount(indexA) == 2 );
    QVERIFY (model->rowCount(indexB) == 0 );
    QVERIFY (model->rowCount(indexC) == 0 );
    QVERIFY (model->rowCount(indexD) == 2 );
    QVERIFY (model->rowCount(indexE) == 0 );
    QVERIFY (model->rowCount(indexF) == 0 );
    QVERIFY (!(QModelIndex().isValid()));
    QVERIFY (!(indexG.isValid()));
    
}

void Asn1BerModelTest::testParentIndices()
{
    Asn1BerNode a;
    Asn1BerNode b(&a);
    Asn1BerNode c(&b);
    Asn1BerNode d;
    c.insertAfter(&d);
    Asn1BerNode e(&d);
    
    Asn1BerModel* model = new Asn1BerModel(&a);
    
    QModelIndex indexA = model->index( 0, 0, QModelIndex());;
    QModelIndex indexB = model->index( 0, 0, indexA);
    QModelIndex indexC = model->index( 0, 0, indexB);
    QModelIndex indexD = model->index( 1, 0, indexB);
    QModelIndex indexE = model->index( 0, 0, indexD);
    
    QVERIFY ( indexA.isValid() );
    QVERIFY ( indexB.isValid() );
    QVERIFY ( indexC.isValid() );
    QVERIFY ( indexD.isValid() );
    QVERIFY ( indexE.isValid() );
    
    QVERIFY ( indexA.parent() == QModelIndex() );
    QVERIFY ( indexB.parent() == indexA );
    QVERIFY ( indexC.parent() == indexB );
    QVERIFY ( indexD.parent() == indexB );
    QVERIFY ( indexE.parent() == indexD );
}

void Asn1BerModelTest::testNavigateIndices()
{
    QByteArray stream;
    
    stream.append((char) 0x30); // Universal [16] (sequence/sequence of)
    stream.append((char) UNKNOWN_LENGTH);
    stream.append((char) 0x30); // Universal [16] (sequence/sequence of)
    stream.append((char) UNKNOWN_LENGTH);
    stream.append((char) 0x40); // Application [0]
    stream.append((char) 0x01); // length = 1
    stream.append('a');
    stream.append((char) 0x41); // Application [1]
    stream.append((char) 0x01); // length = 1
    stream.append('b');
    stream.append((char) 0); // end-of-contents (2 octets)
    stream.append((char) 0);
    stream.append((char) 0x30); // Universal [16] (sequence/sequence of)
    stream.append((char) UNKNOWN_LENGTH);
    stream.append((char) 0x42); // Application [2]
    stream.append((char) 0x03); // length = 3
    stream.append('a');
    stream.append('c');
    stream.append('r');
    stream.append((char) 0x43); // Application [3]
    stream.append((char) 0x01); // length = 1
    stream.append('y');
    stream.append((char) 0); // end-of-contents (2 octets)
    stream.append((char) 0);
    stream.append((char) 0); // end-of-contents (2 octets)
    stream.append((char) 0);

    Asn1BerParser* parser = new Asn1BerParser(stream);
    Asn1BerNode* node;
    
    node = parser->parse();

    Asn1BerModel* model = new Asn1BerModel(node);

    QModelIndex indexA = model->index( 0, 0, QModelIndex());;
    QModelIndex indexB = model->index( 0, 0, indexA);
    QModelIndex indexC = model->index( 0, 0, indexB);
    
    qDebug() << (model->data(indexA, Qt::DisplayRole));
    qDebug() << (model->data(indexB, Qt::DisplayRole));
    qDebug() << (model->data(indexC, Qt::DisplayRole));

    QVERIFY (model->data(indexA, Qt::DisplayRole) == QString("UNIVERSAL [16]"));
    QVERIFY (model->data(indexB, Qt::DisplayRole) == QString("UNIVERSAL [16]"));
    QVERIFY (model->data(indexC, Qt::DisplayRole) == QString("APPLICATION [0]"));

}

void Asn1BerModelTest::cleanupTestCase()
{
    qDebug() << "cleanup";
}

// this thing defines a standard main function
QTEST_MAIN(Asn1BerModelTest)

#include "modeltest.moc"